library namebase;

import 'package:get/get.dart';
import 'package:namebase/src/api_controller.dart';
import 'package:namebase/src/models/record.dart';
import 'package:namebase/src/models/response.dart';
import 'src/models/status.dart';

export 'src/models/record.dart';
export 'src/models/response.dart';
export 'src/models/result.dart';
export 'src/models/status.dart';
export 'src/api_controller.dart';

/// Class Manages Namebase DNS Entries
class Namebase {
  static late String? userName;

  static bool get hasUser => userName != null;

  // Internal Reference
  static late final NamebaseApi _internal;

  Namebase({required String key, required String secret, String? userName}) {
    _internal = NamebaseApi(key: key, secret: secret);
    if (userName != null) {
      Namebase.userName = userName;
    }
  }

  /// Add Records to DNS Table
  static Future<bool> addRecords(List<HSRecord> records) => _internal.addRecords(records);

  /// Find Matching Peer for SName Query of Record
  static Future<HSRecord> findRecord(String query) async {
    final result = await refresh();
    final record = result.records.firstWhere(
      (e) => e.host.toLowerCase() == query.toLowerCase(),
      orElse: () => HSRecord.blank(),
    );
    return record;
  }

  /// Check if Auth Record and SName Record Exists
  static Future<bool> hasAllRecords() async {
    return await hasAuthRecord() && await hasSNameRecord();
  }

  /// Check if Auth Record Exists
  static Future<bool> hasAuthRecord() async {
    final list = await userRecords;
    return list.any((e) => e.isAuth);
  }

  /// Check if SName Record Exists
  static Future<bool> hasSNameRecord() async {
    final list = await userRecords;
    return list.any((e) => e.isName);
  }

  /// Replace Record
  static Future<Response<dynamic>?> replaceRecord(HSRecord record) => _internal.replaceRecord(record);

  /// Refresh Records with Result
  static Future<NamebaseResponse> refresh() => _internal.refresh();

  /// Find Records for User
  static Future<List<HSRecord>> get userRecords async {
    // Refresh Records
    final result = await refresh();

    // Check Username set
    if (hasUser) {
      final records = result.records.where(
        (e) => e.host.toLowerCase() == userName || e.name.toLowerCase() == userName,
      );
      return records.toList();
    } else {
      final records = result.records.where(
        (e) => e.host.toLowerCase() == userName || e.name.toLowerCase() == userName,
      );
      return records.toList();
    }
  }

  /// Validates SName as Valid characters
  static Future<NameStatus> validateName(String sName) async {
    final result = await refresh();
    // Check Alphabet Only
    if (!sName.isAlphabetOnly) {
      return NameStatus.InvalidCharacters;
    } else {
      // Update Status
      if (sName.length > 3) {
        // Check Available
        if (result.checkName(NameCheckType.Unavailable, sName)) {
          return NameStatus.Unavailable;
        }
        // Check Unblocked
        else if (result.checkName(NameCheckType.Blocked, sName)) {
          return NameStatus.Blocked;
        }
        // Check Unrestricted
        else if (result.checkName(NameCheckType.Restricted, sName)) {
          return NameStatus.Restricted;
        }
        // Check Valid
        else {
          return NameStatus.Available;
        }
      } else {
        return NameStatus.TooShort;
      }
    }
  }
}
