import 'dart:convert';
import 'package:get/get.dart';
import 'package:namebase/namebase.dart';

import 'models/record.dart';
import 'models/response.dart';

/// #### Namebase API Client
class NamebaseApi extends GetConnect {
  // Key/Secret Params
  final String key;
  final String secret;

  // Base Client Properties
  final _baseUrl = "https://www.namebase.io";
  String get _authorization => base64Encode(utf8.encode("$key:$secret"));

  // Endpoint for Nameserver DNS Settings
  final _nambaseDnsPoint = '/api/v0/dns/domains/snr/nameserver';

  NamebaseApi({required this.key, required this.secret});

  // Authorization Header Map
  Map<String, String> get _authHeaders =>
      {'Authorization': 'Basic $_authorization', 'Accept': 'application/json', 'Content-Type': 'application/json'};

  /// Returns All Records List
  Future<NamebaseResponse> refresh() async {
    var resp = await get(_baseUrl + _nambaseDnsPoint, headers: _authHeaders);
    return NamebaseResponse.fromResponse(resp);
  }

  /// Method Creates New Item on DNS Table
  Future<bool> addRecords(List<HSRecord> records) async {
    // Convert Records to Map List
    final List<Map<String, dynamic>> mappedRecords = records.map<Map<String, dynamic>>((e) => e.toMap()).toList();

    // Build Request
    var map = {'records': mappedRecords, 'deleteRecords': []};
    var body = jsonEncode(map);
    var resp = await put(_baseUrl + _nambaseDnsPoint, body, headers: _authHeaders);
    return resp.body != null ? resp.body["success"] : true;
  }

  /// Method Replaces Existing Item with New Item on DNS Table
  Future<Response<dynamic>?> replaceRecord(HSRecord record) async {
    // Fetch Data
    final data = await refresh();
    final oldRecord = data.records.firstWhere(
      (r) => r.equalsName(r.host),
      orElse: () => HSRecord.blank(),
    );

    // Remove Old Record
    var deleteMap = {
      'records': [],
      'deleteRecords': [oldRecord.toMap()]
    };

    // Perform Delete Request
    var deleteBody = jsonEncode(deleteMap);
    await put(_baseUrl + _nambaseDnsPoint, deleteBody, headers: _authHeaders);

    // Place New Record
    var addMap = {
      'records': [record.toMap()],
      'deleteRecords': []
    };

    // Perform Add Request
    var addBody = jsonEncode(addMap);
    return await put(_baseUrl + _nambaseDnsPoint, addBody, headers: _authHeaders);
  }

  /// Method Removes List of DNS Records
  Future<bool> removeRecords(List<HSRecord> records) async {
    // Convert Records to Map List
    final List<Map<String, dynamic>> mappedRecords = records.map<Map<String, dynamic>>((e) => e.toMap()).toList();

    // Build Request
    var map = {'records': [], 'deleteRecords': mappedRecords};
    var body = jsonEncode(map);
    var resp = await put(_baseUrl + _nambaseDnsPoint, body, headers: _authHeaders);
    return resp.body != null ? resp.body["success"] : true;
  }
}
