import 'dart:typed_data';

const FINGERPRINT_DIVIDER = "v=0;fingerprint=";
const AUTH_DIVIDER = "._auth.";

class HSRecord {
  HSRecord({
    required this.ttl,
    required this.type,
    required this.host,
    required this.value,
  });

  int ttl;
  String type;
  String host;
  String value;

  bool get isAuth => this.host.contains(AUTH_DIVIDER);
  bool get isName => !this.isAuth && !this.isBlank && !this.host.contains("_") && this.type.toLowerCase().contains("txt");
  bool get isBlank => this.ttl == -1 && this.type == "-" && this.host == "-" && this.value == "-";
  bool get isNotBlank => !this.isBlank;
  Uint8List get fingerprint => isAuth ? extractFingerprint(value) : Uint8List(0);

  String get prefix => isAuth ? extractPrefix(host) : "";
  String get publicKey => isName ? this.value : "";

  String get name {
    if (isAuth) {
      return extractName(host);
    } else if (isName) {
      return host;
    }
    return "";
  }

  // Method Checks if Prefix Values are the same
  bool equals(String n, String p) => isAuth ? this.equalsName(n) && this.equalsPrefix(p) : false;
  bool equalsName(String n) => this.name.toLowerCase() == n.toLowerCase();
  bool equalsPrefix(String p) => this.prefix == p;
  bool notEquals(String n, String p) => this.notEqualsName(n) && this.notEqualsPrefix(p);
  bool notEqualsName(String n) => this.name.toLowerCase() != n.toLowerCase();
  bool notEqualsPrefix(String p) => this.prefix != p;

  /// Returns Blank Record
  factory HSRecord.blank() {
    return HSRecord(ttl: -1, type: "-", host: "-", value: "-");
  }

  /// Creates Record From JSON
  factory HSRecord.fromJson(dynamic json) => HSRecord(
        ttl: json["ttl"],
        type: json["type"],
        host: json["host"],
        value: json["value"],
      );

  /// Creates New `Auth` Record from AuthResponse
  factory HSRecord.newAuth({required String prefix, required String name, required String fingerprint}) {
    return HSRecord(
      ttl: 5,
      type: "TXT",
      host: "$prefix$AUTH_DIVIDER$name",
      value: FINGERPRINT_DIVIDER + fingerprint,
    );
  }

  /// Creates New `Name` Record from AuthResponse
  factory HSRecord.newName({required String publicKey, required String name}) {
    return HSRecord(
      ttl: 5,
      type: "TXT",
      host: name,
      value: publicKey,
    );
  }

  /// Returns Auth/Name Record from AuthResponse
  /// [0] - AuthRecord
  /// [1] - NameRecord
  static List<HSRecord> newRegisteredRecords({
    required String prefix,
    required String name,
    required String fingerprint,
    required String publicKey,
  }) =>
      [
        HSRecord.newAuth(prefix: prefix, name: name, fingerprint: fingerprint),
        HSRecord.newName(publicKey: publicKey, name: name),
      ];

  // Extracts FingerPrint from Record Value
  static Uint8List extractFingerprint(String value) {
    var data = value.substring(FINGERPRINT_DIVIDER.length);
    return Uint8List.fromList(data.codeUnits);
  }

  // Extracts Prefix from Record Host
  static String extractPrefix(String host) {
    var idx = host.indexOf(AUTH_DIVIDER);
    return host.substring(0, idx);
  }

  // Extracts Name from Record Host
  static String extractName(String host) {
    var idx = host.indexOf(AUTH_DIVIDER);
    return host.substring(idx + AUTH_DIVIDER.length);
  }

  /// Converts Record to Map for REST API
  Map<String, dynamic> toMap() => {
        "ttl": ttl,
        "type": type,
        "host": host,
        "value": value,
      };

  @override
  String toString() {
    if (this.isAuth) {
      return "--Auth Record-- \n" +
          {
            "Fingerprint": this.fingerprint,
            "Prefix": this.prefix,
            "Name": this.name,
          }.toString();
    } else {
      return "--Default Record-- \n" + this.toMap().toString();
    }
  }
}
